%{

// Directives to the compiler and C declarations
#include <stdio.h>
#define YYERROR_VERBOSE 1

// The lexer
extern int yylex();
extern int yyerror(char *);

%}

// Tokens, given by lexer
%token NUMBER

/* TYPES FOR TOKENS AND RULES
We can associate a SEMANTIC VALUE TO THE RULES OR TOKEN. If we associate a semantic value to a rule or a token, we must
specify the TYPE of that value. 
The variable yylval is used by the scanner (file generated by Flex) to update the semantic value of matched tokens.
Moreover, and it is VERY IMPORTANT, consider a generic rule of a generic grammar: A -> E , A B

Suppose that, A,B and E are rules associated to semantic values. When we write the semantic action of the rule, we surely
want to do something with the semantic values associated to its parts. So, we can refer to each element of the rule using
a positional strategy and some predefined variables, provided by Bison and descrived below.

The 'A' on the left hand side is called $$. 
E is called $1
, is called $2
The 'A' on the right hand side is called $3
B is called $4

Their types are the types sopecified in type definition section 
*/
%type <ival> NUMBER

/* Possible types of rules and token */
%union {
  int	ival;
}

%%

List
  : Par
    { }
  | Par ',' List
    { }
  ;

Par
  : '(' Par ')' 
    { }
  | '(' Par ')' Par
    { }
  | NUMBER Par
    { /* Print the number */
      printf("%d ",$1); }
  | NUMBER
    { /* Print the number.
         Please refer to the flex file to understand how the variable yylval (used to update the semantic values) is
         updated by the scanner */
      printf("%d ",$1); }
  ;
  

