/* syntaxTree.h
 * Data structures and prototypes to handle a very simple syntax tree */

#ifndef ST_H
#define ST_H

/* The syntax tree we want to generate, will be built by composition of nodes. Each node is
 * an instance of a structure stNode.
 * However each node should have some particular characteristics that differntiate it from the others.
 * For example, the nodes associated to the Names of variables should hold the symbolic names; the nodes
 * associated to the grammar rule 'Type' should hold a type identificator, and so on.
 *
 * The question is: how can we differentiate each node? If we are programming in C++, we could use a 
 * superclass (for example, the superclass Node) and its extensions (for example, NodeType, NodeName, 
 * NodeAssignment, NodeVarList, etc etc).
 *
 * The same concepts can be applied to C-programming language, by using void pointers and type casting.
 * I've introduced two fields of the struct stNode. The first one is a void pointer, that points to a data
 * structure that characterizes the node. In this simple example, we have only one kind of 
 * characterization: symbol nodes (SEE FILE SYMBOLFACTORY.H).
 * 
 * The second question is: how can we eventually print the information hold by the data structure pointed by
 * extDat?
 * If we are programming in C++, we simply call the printing methods associated to the subclasses.
 *
 * In this case (using C) we use a pointer to a generic printing function. The right function must be set
 * depending on the specific case. 
 *
 * Please refer to files: symbolFactory.h, symbolFactory.c.
 * 
 */

typedef struct _stNode{
  /************************************/
  void *extDat;
  void (*extPrn)(void *);
  /***********************************/
  
  char *name;
  int nChildren;
  struct _child *children;
}stNode;

typedef struct _child{
  stNode *to;
  struct _child *next;
}child;

//===============================================================================

stNode *createStNode(char *name);
void appendChild(stNode *parent, stNode *chld);

// recursive Pretty Printing
void recursivePP(stNode *root, int ind);

#endif
