%{

// Directives to the compiler and C declarations
#include <stdio.h>
#include "./lib/syntaxTree.h"
#include "./lib/symbolFactory.h"
#define YYERROR_VERBOSE 1

// The lexer
extern int yylex();
extern int yyerror(char *);

// The syntax tree
extern stNode *tree;

%}

// Tokens, given by the lexer
%token IDENTIFIER
%token NUMBER
%token INT
%token FLOAT
%token bEGIN
%token END
%token VARDEC

%type <nodep> Body
%type <nodep> StatementList
%type <nodep> VarDec
%type <nodep> VarList
%type <nodep> NameList
%type <nodep> Name
%type <nodep> Statement
%type <nodep> Assignment
%type <nodep> Expr
%type <nodep> Term
%type <nodep> Factor
%type <str> IDENTIFIER

// Possible types of yylval
%union {
  struct _stNode *nodep;
  char *str;
}

%%

Program
  : bEGIN ProgramName Body END ProgramName
    { tree = createStNode("Program"); 
      appendChild(tree,$3);
    }
  ;

Body
  : VarDec ';' StatementList
    { $$ = createStNode("Body");
      appendChild($$,$3);
      appendChild($$,$1);
    }
  ;

VarDec
  : VARDEC VarList
    { $$ = createStNode("VarDec");
      appendChild($$,$2);
    }
  ;

VarList
  : Type NameList
    { // Remember the LR parsing...how we create here the node?
      $$ = createStNode("VarList"); 
      appendChild($$,$2);
    }
  | Type NameList VarList
    { appendChild($3,$2); 
      $$ = $3;
    }
  ;

NameList
  : Name
    { $$ = createStNode("NameList");
      appendChild($$,$1);
    }
  | Name ',' NameList
    { appendChild($3,$1); 
      $$ = $3;
    }
  ;

StatementList
  : Statement ';'
    { $$ = createStNode("StatementList");
      appendChild($$,$1);
    }
  | Statement ';' StatementList
    { appendChild($3,$1); 
      $$ = $3;
    }
  ;

Statement
  : Assignment
    { $$ = createStNode("Statement");
      appendChild($$,$1);
    }
  ;

Assignment
  : Name '=' Expr
    { $$ = createStNode("Assignment");
      appendChild($$,$1);
      appendChild($$,$3);
    }
  ;

Expr
  : Term '+' Expr
    { $$ = createStNode("Expression"); 
      appendChild($$,$1);
      appendChild($$,$3);
    }
  | Term '-' Expr
    { $$ = createStNode("Expression");
      appendChild($$,$1);
      appendChild($$,$3);
    }
  | Term
    { $$ = createStNode("Expression");
      appendChild($$,$1);
    }
  ;

Term
  : Term '*' Factor
    { $$ = createStNode("Term");
      appendChild($$,$1);
      appendChild($$,$3);
    }
  | Factor
    { $$ = createStNode("Term");
      appendChild($$,$1);
    }
  ;

Factor
  : '(' Expr ')'
    { $$ = createStNode("Factor");
      appendChild($$,$2);
    }
  | NUMBER
    { $$ = createStNode("Factor"); }
  | Name
    { $$ = createStNode("Factor"); 
      appendChild($$,$1);
    }
  ;

ProgramName
  : IDENTIFIER
    { }
  ;

Name
  : IDENTIFIER
    { $$ = createStNode("Name"); 
      /* CHARACTERIZE THE NODE!!! IT IS A SYMBOL NODE!! */
      $$->extDat = (void *)createSymbol($1);
      $$->extPrn = printSymbol;
    }
  ;

Type
  : INT 
  | FLOAT
  ;

  

